indexing

	description:
		"Can be an entire MIME message, or a part of a multipart message."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #1 $"


class

	EPX_MIME_PART


inherit

	STDC_BASE

	EPX_MIME_FIELD_NAMES

creation

	make_empty


feature -- Initialization

	make_empty is
			-- Make an empty MIME part. Useful during parsing to
			-- construct the message gradually.
		do
			create fields.make (64)
		end


feature -- Public state

	fields: DS_HASH_TABLE [EPX_MIME_FIELD, STRING]
			-- All fields that appeared in this part in the MIME message.

	body: EPX_MIME_BODY
			-- The body, can be multipart.


feature -- Certain fields

	content_disposition: EPX_MIME_FIELD_CONTENT_DISPOSITION is
			-- Field `Content-Disposition' if it exists, else Void.
		do
			fields.search (field_name_content_disposition)
			if fields.found then
				Result ?= fields.found_item
			end
		end

	content_type: EPX_MIME_FIELD_CONTENT_TYPE is
			-- Field `Content-Type' if it exists, else Void.
		do
			fields.search (field_name_content_type)
			if fields.found then
				Result ?= fields.found_item
			end
		ensure
			multipart_implies_content_type:
				body /= Void implies (body.is_multipart implies Result /= Void)
		end


feature -- Content-Type parameter names

	parameter_name_boundary: STRING is "boundary"


feature -- Content-Disposition parameter name

	parameter_filename: STRING is "filename"


feature -- Body creation/removal

	clear_body is
			-- Set `body' to Void.
		do
			body := Void
		ensure
			body_void: body = Void
		end

	create_multipart_body is
			-- Set `body' to a container.
		require
			body_not_set: body = Void
		do
			create {EPX_MIME_BODY_MULTIPART} body.make
		ensure
			body_not_void: body /= Void
			body_is_multipart: body.is_multipart
		end

	create_singlepart_body is
			-- Set `body' to be a single part one. If we find a
			-- Content-Disposition field with value form-data, the data
			-- is saved to a file.
		require
			body_not_set: body = Void
		local
			cd: EPX_MIME_FIELD_CONTENT_DISPOSITION
		do
			cd := content_disposition
			if cd = Void then
				create {EPX_MIME_BODY_STRING} body.make
			else
				if cd.parameters.has (parameter_filename) then
					create {EPX_MIME_BODY_FILE} body.make
				else
					create {EPX_MIME_BODY_STRING} body.make
				end
			end
		ensure
			body_not_void: body /= Void
			body_is_singlepart: not body.is_multipart
		end


invariant

	fields_not_void: fields /= Void

end
