indexing

	description: "Abstract fork root class."

	usage: "Implement execute method and either:%
	%- pass this class to your fork method if you inherit from%
	%  POSIX_CURRENT_PROCESS%
	%- or call the fork method and pass yourself."


	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"


deferred class

	POSIX_FORK_ROOT


inherit

	POSIX_CHILD_PROCESS
		rename
			is_pid_valid as is_child_pid_valid
		redefine
			pid
		end

	POSIX_CURRENT_PROCESS
		redefine
			pid
		select
			is_pid_valid
		end

	PAPI_WAIT

	CAPI_STDLIB


feature -- process properties

	pid: INTEGER is
			-- either the current process identifier or the child's
		do
			if my_pid > 0 then
				Result := my_pid
			else
				Result := posix_getpid
			end
		end

	is_valid_child_process: BOOLEAN is
			-- returns True if this object seems to refer to a valid child process
			-- the real child process might have stopped though
		do
			Result := my_pid > 0
		end


feature {POSIX_CURRENT_PROCESS}

	set_running is
			-- Called by parent after successful fork.
		do
			running := True
		ensure
			not_terminated: not is_terminated
		end

	frozen start is
			-- Called by the `fork' routine.
			-- Makes sure child does never return.
		do
			after_fork
			execute
			exit_with_success
		rescue
			-- last resort
			minimal_exit (EXIT_FAILURE)
		end


feature -- deferred routines

	after_fork is
			-- chance for code to do something before the main execute
			-- mainly here for POSIX_DAEMON.
		do
			-- do nothing
		end

	execute is
			-- Start if child process.
		deferred
		end


feature -- termination info

	is_terminated_normally,
	is_exited: BOOLEAN is
			-- Has this process been terminated normally?
		require
			valid_status_info: is_terminated
		do
			Result := posix_wifexited (termination_info)
		end

	exit_code: INTEGER is
			-- Low-order 8 bits of call to _exit or exit for this process.
		require else
			terminated_normally: is_terminated_normally
		do
			Result := posix_wexitstatus (termination_info)
		end

	is_signalled: BOOLEAN is
			-- Child process was terminated due to receipt of a signal
			-- that was not caught.
		require
			valid_status_info: is_terminated
		do
			Result := posix_wifsignaled (termination_info)
		end

	signal_code: INTEGER is
			-- Signal of process terminated abnormally or was stopped.
		require
			valid_status_info: is_terminated
			terminated_by_signal: is_signalled
		do
			Result := posix_wtermsig (termination_info)
		end


end
