indexing

	description: "Class that covers the Standard C file system code. %
	%There was not much to cover..."

	author: "Berend de Boer"
	date: "$Date: 2003/01/09 $"
	revision: "$Revision: #3 $"

class

	STDC_FILE_SYSTEM

inherit

	STDC_BASE

	PORTABLE_PATH

	CAPI_STDIO


feature -- path names

	expand_path (a_path: STRING): STDC_PATH is
			-- returns a new path
		do
			create Result.make_expand (a_path)
		end


feature -- rename files/directories, remove files/directories

	remove_file (a_path: STRING) is
			-- Removes a file from a directory.
			-- For Standard C, it's implementation defined what
			-- remove_file does if file is opened by some process
			-- (`remove_file' fails on Windows for example).
			-- doesn't remove a directory.
		require
			valid_path: a_path /= Void and then not a_path.is_empty
		local
			r: INTEGER
		do
			set_portable_path (a_path)
			r := posix_remove (sh.string_to_pointer (portable_path))
			-- in case remove failed because the file did no longer exist,
			-- we do not want to raise an exception. That's why we check
			-- if the file still exists.
			if r = -1 and then is_readable (a_path) then
				-- if it exists, try to remove it again to set error code
				r := posix_remove (sh.string_to_pointer (portable_path))
				if r /= 0 then
					raise_posix_error
				end
			end
		end

	rename_to (current_path, new_path: STRING) is
			-- Rename a file or a directory.
			-- `new_path' should not be an existing path.
		require
			valid_current: current_path /= Void and then not current_path.is_empty
			valid_new: new_path /= Void and then not new_path.is_empty
		local
			c_current_path,
			c_new_path: POINTER
			the_new_path: STDC_PATH
		do
			set_portable_path (current_path)
			c_current_path := sh.string_to_pointer (portable_path)
			create the_new_path.make_from_string (new_path)
			c_new_path := sh.string_to_pointer (the_new_path)
			safe_call (posix_rename (c_current_path, c_new_path))
		end


feature -- accessibility of files

	is_modifiable (a_path: STRING): BOOLEAN is
			-- Is `a_path' readable and writable by this program?
			-- Does this by attemting to open `a_path' file read/write.
		require
			valid_path: a_path /= Void and then not a_path.is_empty
		local
			file: STDC_BINARY_FILE
			oops: BOOLEAN
		do
			if oops then
				Result := False
			else
				create file.open_read_write (a_path)
				file.close
				Result := True
			end
		rescue
			oops := True
			retry
		end

	is_readable (a_path: STRING): BOOLEAN is
			-- tests if file is readable by this program
			-- does this by attemting to open `a_path' file read-only
		require
			valid_path: a_path /= Void and then not a_path.is_empty
		local
			file: STDC_BINARY_FILE
			oops: BOOLEAN
		do
			if oops then
				Result := False
			else
				create file.open_read (a_path)
				file.close
				Result := True
			end
		rescue
			oops := True
			retry
		end


end
